@@ -110,10 +110,27 @@ extern unsigned int __unsigned_int_flags_dummy;
 					} while (0)
 
 
-/* The following 64-bit flag code can most likely be erased after app_dial
-   is reorganized to either reduce the large number of options, or handle
-   them in some other way. At the time of this writing, app_dial would be
-   the only user of 64-bit option flags */
+/*!
+ * \brief Swap the upper and lower 32 bits of a big-endian 64-bit integer
+ *
+ * This macro is needed to preserve ABI compatability on big-endian systems
+ * after changing from a 32 bit flags to a 64 bit flags.  It ensures that a
+ * new 64-bit flag field will still work with a function that expects a
+ * 32-bit flag field.  On a little-endian system, nothing is needed, since
+ * the 64-bit flags are already in the correct order.
+ *
+ * \note This macro is different than a standard byte swap, as it
+ * doesn't reverse the byte order, it just swaps the upper 4 bytes with
+ * the lower 4 bytes.
+ *
+ * \param flags The 64-bit flags to swap
+ * \retval The flags with the upper and lower 32 bits swapped if the system is big-endian,
+ */
+#if __BYTE_ORDER == __BIG_ENDIAN
+#define SWAP64_32(flags) (((uint64_t)flags << 32) | ((uint64_t)flags >> 32))
+#else
+#define SWAP64_32(flags) (flags)
+#endif
 
 extern uint64_t __unsigned_int_flags_dummy64;
 
@@ -121,21 +138,21 @@ extern uint64_t __unsigned_int_flags_dummy64;
 					typeof ((p)->flags) __p = (p)->flags; \
 					typeof (__unsigned_int_flags_dummy64) __x = 0; \
 					(void) (&__p == &__x); \
-					((p)->flags & (flag)); \
+					((p)->flags & SWAP64_32(flag)); \
 					})
 
 #define ast_set_flag64(p,flag) 		do { \
 					typeof ((p)->flags) __p = (p)->flags; \
 					typeof (__unsigned_int_flags_dummy64) __x = 0; \
 					(void) (&__p == &__x); \
-					((p)->flags |= (flag)); \
+					((p)->flags |= SWAP64_32(flag)); \
 					} while(0)
 
 #define ast_clear_flag64(p,flag) 		do { \
 					typeof ((p)->flags) __p = (p)->flags; \
 					typeof (__unsigned_int_flags_dummy64) __x = 0; \
 					(void) (&__p == &__x); \
-					((p)->flags &= ~(flag)); \
+					((p)->flags &= ~SWAP64_32(flag)); \
 					} while(0)
 
 #define ast_copy_flags64(dest,src,flagz)	do { \
@@ -144,8 +161,8 @@ extern uint64_t __unsigned_int_flags_dummy64;
 					typeof (__unsigned_int_flags_dummy64) __x = 0; \
 					(void) (&__d == &__x); \
 					(void) (&__s == &__x); \
-					(dest)->flags &= ~(flagz); \
-					(dest)->flags |= ((src)->flags & (flagz)); \
+					(dest)->flags &= ~SWAP64_32(flagz); \
+					(dest)->flags |= ((src)->flags & SWAP64_32(flagz)); \
 					} while (0)
 
 #define ast_set2_flag64(p,value,flag)	do { \
@@ -153,19 +170,20 @@ extern uint64_t __unsigned_int_flags_dummy64;
 					typeof (__unsigned_int_flags_dummy64) __x = 0; \
 					(void) (&__p == &__x); \
 					if (value) \
-						(p)->flags |= (flag); \
+						(p)->flags |= SWAP64_32(flag); \
 					else \
-						(p)->flags &= ~(flag); \
+						(p)->flags &= ~SWAP64_32(flag); \
 					} while (0)
 
 #define ast_set_flags_to64(p,flag,value)	do { \
 					typeof ((p)->flags) __p = (p)->flags; \
 					typeof (__unsigned_int_flags_dummy64) __x = 0; \
 					(void) (&__p == &__x); \
-					(p)->flags &= ~(flag); \
-					(p)->flags |= (value); \
+					(p)->flags &= ~SWAP64_32(flag); \
+					(p)->flags |= SWAP64_32(value); \
 					} while (0)
 
+#define AST_FLAGS64_ALL ULONG_MAX
 
 /* Non-type checking variations for non-unsigned int flags.  You
    should only use non-unsigned int flags where required by
@@ -248,7 +266,7 @@ int ast_base64encode(char *dst, const unsigned char *src, int srclen, int max);
  * \param src The source buffer
  *
  * \retval NULL on failure
- * \retval Encoded string on success
+ * \return Encoded string on success
  */
 char *ast_base64encode_string(const char *src);
 
@@ -272,7 +290,7 @@ int ast_base64decode(unsigned char *dst, const char *src, int max);
  * \param src The source buffer
  *
  * \retval NULL on failure
- * \retval Decoded string on success
+ * \return Decoded string on success
  */
 char *ast_base64decode_string(const char *src);
 
@@ -320,7 +338,7 @@ int ast_base64url_encode(char *dst, const unsigned char *src, int srclen, int ma
  * \param src The source buffer
  *
  * \retval NULL on failure
- * \retval Decoded string on success
+ * \return Decoded string on success
  */
 char *ast_base64url_decode_string(const char *src);
 
@@ -332,10 +350,30 @@ char *ast_base64url_decode_string(const char *src);
  * \param src The source data to be encoded
  *
  * \retval NULL on failure
- * \retval Encoded string on success
+ * \return Encoded string on success
  */
 char *ast_base64url_encode_string(const char *src);
 
+/*!
+ * \brief Performs a base 64 encode algorithm on the contents of a File
+ * \param inputfile A FILE handle to the input file to be encoded. Must be readable. This handle is not automatically closed.
+ * \param outputfile A FILE handle to the output file to receive the base 64 encoded contents of the input file, identified by filename.
+ * \param endl The line ending to use (e.g. either "\n" or "\r\n")
+ *
+ * \return zero on success, -1 on error.
+ */
+int ast_base64_encode_file(FILE *inputfile, FILE *outputfile, const char *endl);
+
+/*!
+ * \brief Performs a base 64 encode algorithm on the contents of a File
+ * \param filename The path to the file to be encoded. Must be readable, file is opened in read mode.
+ * \param outputfile A FILE handle to the output file to receive the base 64 encoded contents of the input file, identified by filename.
+ * \param endl The line ending to use (e.g. either "\n" or "\r\n")
+ *
+ * \return zero on success, -1 on error.
+ */
+int ast_base64_encode_file_path(const char *filename, FILE *outputfile, const char *endl);
+
 #define AST_URI_ALPHANUM     (1 << 0)
 #define AST_URI_MARK         (1 << 1)
 #define AST_URI_UNRESERVED   (AST_URI_ALPHANUM | AST_URI_MARK)
@@ -376,6 +414,19 @@ char *ast_uri_encode(const char *string, char *outbuf, int buflen, struct ast_fl
  */
 void ast_uri_decode(char *s, struct ast_flags spec);
 
+/*!
+ * \brief Verify if a string is valid as a URI component
+ *
+ * This function checks if the string either doesn't need encoding
+ * or is already properly URI encoded.
+ * Valid characters are 'a-zA-Z0-9.+_-' and '%xx' escape sequences.
+ *
+ * \param string String to be checked
+ * \retval 1 if the string is valid
+ * \retval 0 if the string is not valid
+ */
+int ast_uri_verify_encoded(const char *string);
+
 /*! ast_xml_escape
 	\brief Escape reserved characters for use in XML.
 
@@ -385,8 +436,8 @@ void ast_uri_decode(char *s, struct ast_flags spec);
 	\param string String to be converted
 	\param outbuf Resulting encoded string
 	\param buflen Size of output buffer
-	\return 0 for success
-	\return -1 if buflen is too short.
+	\retval 0 for success
+	\retval -1 if buflen is too short.
  */
 int ast_xml_escape(const char *string, char *outbuf, size_t buflen);
 
@@ -470,8 +521,11 @@ static force_inline void ast_slinear_saturated_multiply_float(short *input, floa
 		*input = 32767;
 	else if (res < -32768)
 		*input = -32768;
+	else if (res > 0)
+		*input = (short) (res + 0.5);
 	else
-		*input = (short) res;
+		*input = (short) (res - 0.5);
+
 }
 
 static force_inline void ast_slinear_saturated_divide(short *input, short *value)
@@ -486,8 +540,11 @@ static force_inline void ast_slinear_saturated_divide_float(short *input, float
 		*input = 32767;
 	else if (res < -32768)
 		*input = -32768;
+	else if (res > 0)
+		*input = (short) (res + 0.5);
 	else
-		*input = (short) res;
+		*input = (short) (res - 0.5);
+
 }
 
 #ifdef localtime_r
@@ -601,7 +658,6 @@ long int ast_random(void);
 /*!
  * \brief Disable PMTU discovery on a socket
  * \param sock The socket to manipulate
- * \return Nothing
  *
  * On Linux, UDP sockets default to sending packets with the Dont Fragment (DF)
  * bit set. This is supposedly done to allow the application to do PMTU
@@ -617,7 +673,8 @@ void ast_enable_packet_fragmentation(int sock);
  * \brief Recursively create directory path
  * \param path The directory path to create
  * \param mode The permissions with which to try to create the directory
- * \return 0 on success or an error code otherwise
+ * \retval 0 on success
+ * \return error code otherwise
  *
  * Creates a directory path, creating parent directories as needed.
  */
@@ -630,9 +687,11 @@ int ast_mkdir(const char *path, int mode);
  * If \a base_path does not exist, it will not be created and this function
  * returns \c EPERM.
  *
+ * \param base_path
  * \param path The directory path to create
  * \param mode The permissions with which to try to create the directory
- * \return 0 on success or an error code otherwise
+ * \retval 0 on success
+ * \return an error code otherwise
  */
 int ast_safe_mkdir(const char *base_path, const char *path, int mode);
 
@@ -644,7 +703,8 @@ int ast_safe_mkdir(const char *base_path, const char *path, int mode);
  * \param v the value to check
  * \param min minimum lower bound (inclusive)
  * \param max maximum upper bound (inclusive)
- * \return 0 if value out of bounds, otherwise true (non-zero)
+ * \retval 0 if value out of bounds
+ * \retval non-zero otherwise
  */
 #define IN_BOUNDS(v, min, max) ((v) >= (min)) && ((v) <= (max))
 
@@ -653,7 +713,8 @@ int ast_safe_mkdir(const char *base_path, const char *path, int mode);
  *
  * \param v the value to check
  * \param a the array to bound check
- * \return 0 if value out of bounds, otherwise true (non-zero)
+ * \retval 0 if value out of bounds
+ * \retval non-zero otherwise
  */
 #define ARRAY_IN_BOUNDS(v, a) IN_BOUNDS((int) (v), 0, ARRAY_LEN(a) - 1)
 
@@ -675,7 +736,7 @@ struct ast_http_digest {
 
 /*!
  * \brief Parse digest authorization header.
- * \return Returns -1 if we have no auth or something wrong with digest.
+ * \return -1 if we have no auth or something wrong with digest.
  * \note This function may be used for Digest request and responce header.
  * request arg is set to nonzero, if we parse Digest Request.
  * pedantic arg can be set to nonzero if we need to do addition Digest check.
@@ -720,8 +781,6 @@ static void force_inline _ast_assert(int condition, const char *condition_str, c
  * \brief Force a crash if DO_CRASH is defined.
  *
  * \note If DO_CRASH is not defined then the function returns.
- *
- * \return Nothing
  */
 void DO_CRASH_NORETURN ast_do_crash(void);
 
@@ -813,7 +872,8 @@ char *ast_eid_to_str(char *s, int maxlen, struct ast_eid *eid);
  * This function expects an EID in the format:
  *    00:11:22:33:44:55
  *
- * \return 0 success, non-zero failure
+ * \retval 0 success
+ * \retval non-zero failure
  * \since 1.6.1
  */
 int ast_str_to_eid(struct ast_eid *eid, const char *s);
@@ -821,7 +881,8 @@ int ast_str_to_eid(struct ast_eid *eid, const char *s);
 /*!
  * \brief Compare two EIDs
  *
- * \return 0 if the two are the same, non-zero otherwise
+ * \retval 0 if the two are the same
+ * \retval non-zero otherwise
  * \since 1.6.1
  */
 int ast_eid_cmp(const struct ast_eid *eid1, const struct ast_eid *eid2);
@@ -829,7 +890,8 @@ int ast_eid_cmp(const struct ast_eid *eid1, const struct ast_eid *eid2);
 /*!
  * \brief Check if EID is empty
  *
- * \return 1 if the EID is empty, zero otherwise
+ * \retval 1 if the EID is empty
+ * \retval 0 otherwise
  * \since 13.12.0
  */
 int ast_eid_is_empty(const struct ast_eid *eid);
@@ -929,7 +991,7 @@ static inline void _raii_cleanup_block(_raii_cleanup_block_t *b) { (*b)(); }
  * \param key User's password to crypt.
  * \param salt Salt to crypt with.
  * \return Crypted password.
- * \return \c NULL on error.
+ * \retval NULL on error.
  */
 char *ast_crypt(const char *key, const char *salt);
 
@@ -942,7 +1004,7 @@ char *ast_crypt(const char *key, const char *salt);
  *
  * \param key User's password to crypt.
  * \return Crypted password.
- * \return \c NULL on error.
+ * \retval NULL on error.
  */
 char *ast_crypt_encrypt(const char *key);
 
@@ -951,8 +1013,8 @@ char *ast_crypt_encrypt(const char *key);
  *
  * \param key User's password to validate.
  * \param expected Expected result from crypt.
- * \return True (non-zero) if \a key matches \a expected.
- * \return False (zero) if \a key doesn't match.
+ * \retval True (non-zero) if \a key matches \a expected.
+ * \retval False (zero) if \a key doesn't match.
  */
 int ast_crypt_validate(const char *key, const char *expected);
 
@@ -961,8 +1023,8 @@ int ast_crypt_validate(const char *key, const char *expected);
  * \since 13.7.0
  *
  * \param filename File to test.
- * \return True (non-zero) if the file exists and is readable.
- * \return False (zero) if the file either doesn't exists or is not readable.
+ * \retval True (non-zero) if the file exists and is readable.
+ * \retval False (zero) if the file either doesn't exists or is not readable.
  */
 int ast_file_is_readable(const char *filename);
 
@@ -970,12 +1032,12 @@ int ast_file_is_readable(const char *filename);
  * \brief Compare 2 major.minor.patch.extra version strings.
  * \since 13.7.0
  *
- * \param version1.
- * \param version2.
+ * \param version1
+ * \param version2
  *
- * \return <0 if version 1 < version 2.
- * \return =0 if version 1 = version 2.
- * \return >0 if version 1 > version 2.
+ * \retval negative if version 1 < version 2.
+ * \retval 0 if version 1 = version 2.
+ * \retval positive if version 1 > version 2.
  */
 int ast_compare_versions(const char *version1, const char *version2);
 
@@ -983,8 +1045,8 @@ int ast_compare_versions(const char *version1, const char *version2);
  * \brief Test that an OS supports IPv6 Networking.
  * \since 13.14.0
  *
- * \return True (non-zero) if the IPv6 supported.
- * \return False (zero) if the OS doesn't support IPv6.
+ * \retval True (non-zero) if the IPv6 supported.
+ * \retval False (zero) if the OS doesn't support IPv6.
  */
 int ast_check_ipv6(void);
 
@@ -1003,8 +1065,8 @@ enum ast_fd_flag_operation {
  * \param fd File descriptor to set flags on
  * \param flags The flag(s) to set
  *
- * \return -1 on error
- * \return 0 if successful
+ * \retval -1 on error
+ * \retval 0 if successful
  */
 #define ast_fd_set_flags(fd, flags) \
 	__ast_fd_set_flags((fd), (flags), AST_FD_FLAG_SET, __FILE__, __LINE__, __PRETTY_FUNCTION__)
@@ -1019,8 +1081,8 @@ enum ast_fd_flag_operation {
  * \param fd File descriptor to clear flags on
  * \param flags The flag(s) to clear
  *
- * \return -1 on error
- * \return 0 if successful
+ * \retval -1 on error
+ * \retval 0 if successful
  */
 #define ast_fd_clear_flags(fd, flags) \
 	__ast_fd_set_flags((fd), (flags), AST_FD_FLAG_CLEAR, __FILE__, __LINE__, __PRETTY_FUNCTION__)
@@ -1067,17 +1129,27 @@ int ast_pipe_nonblock(int filedes[2]);
  *
  * \param is_user_interface Non-zero to mark the thread as a user interface.
  *
- * \return 0 if successfuly marked current thread.
- * \return Non-zero if marking current thread failed.
+ * \retval True (non-zero) if marking current thread failed.
+ * \retval False (zero) if successfuly marked current thread.
  */
 int ast_thread_user_interface_set(int is_user_interface);
 
 /*!
  * \brief Indicates whether the current thread is a user interface
  *
- * \return True (non-zero) if thread is a user interface.
- * \return False (zero) if thread is not a user interface.
+ * \retval True (non-zero) if thread is a user interface.
+ * \retval False (zero) if thread is not a user interface.
  */
 int ast_thread_is_user_interface(void);
 
+/*!
+ * \brief Test for the presence of an executable command in $PATH
+ *
+ * \param cmd Name of command to locate.
+ *
+ * \retval True (non-zero) if command is in $PATH.
+ * \retval False (zero) command not found.
+ */
+int ast_check_command_in_path(const char *cmd);
+
 #endif /* _ASTERISK_UTILS_H */
